%% PTrussMatlab (PTrussMatlab.m)
%  Program for the Linear Static Analysis of Plane Trusses with Matlab
%  ------------------------------------------------------------------------
%  v1.00, 1 January 2016
%  (c) Copyright 2016
%  Vagelis Plevris, George Papazafeiropoulos
%  All rights reserved
%  The Program is distributed under a BSD-style license (see License.txt)
%  email: vplevris@gmail.com, gpapazafeiropoulos@yahoo.gr
%  ------------------------------------------------------------------------
%  Files that belong to the program:
%  1. PTrussMatlab.m (this file)
%  2. PTrussDataInput.m
%  3. PTrussAssignElementProperties.m
%  4. PTrussElementLength.m
%  5. PTrussElementStiffness.m
%  6. PTrussElementTransformation.m
%  7. PTrussDataOutput.m
%  8. PTrussDrawModel.m
%
%  Additional files:
%  1. Documentation.pdf: Documentation of the program
%  2. Version History.txt: Version History of the program
%  3. Truss10.txt: Example input file (see below)
%  4. Truss10_Output.txt: Output file of the Example input file
%  5. License.txt: BSD-style license of the program
%
%  An input file is always needed containing the model data
%  One example input file (Truss10.txt) is included in the package
%  When run for Truss10.txt, the program generates the output text file
%  "Model1_Output.txt"


%% Start main program PTrussMatlab.m
clear %Clear memory (workspace)
clc %Clear command window
format long; %Show results with many decimals


%% Get model data and draw model
PTrussDataInput;
tic %Initialize timer for analysis
PTrussDrawModel %Draw model
NumDOFs = 2*NumNodes; %Total Degrees Of Freedom for whole model

%% Form structure's global stiffness matrix
%  KGlobalStructure [NumDOFs x NumDOFs]
KGlobalStructure=zeros(NumDOFs); %Initialize KGlobalStructure putting zeros in it
for n=1:NumElements %Scan all elements
	PTrussAssignElementProperties; %Get properties for n element
    %Form matrices
    KLocal = PTrussElementStiffness(E,A,L); 
	Transform = PTrussElementTransformation(x1,y1,x2,y2); %Form transformation matrix (Transform) for n element [2x4]
	KGlobal = (Transform')*KLocal*Transform ; %Form global stiffness matrix (KGlobal) for n element [4x4]
	DOFi = 2*Node_i-1 ; %x-Degree OF Freedom for start node (i)
	DOFj = 2*Node_j-1 ; %x-Degree OF Freedom for end node (j)
    
	%Add the element's global stiffness matrix [4x4] to the appropriate position in
	%the structure's global stiffness matrix [NumNodes*2 x NumNodes*2]
	KGlobalStructure(DOFi:DOFi+1,DOFi:DOFi+1)=KGlobalStructure(DOFi:DOFi+1,DOFi:DOFi+1)+KGlobal(1:2,1:2);
	KGlobalStructure(DOFj:DOFj+1,DOFj:DOFj+1)=KGlobalStructure(DOFj:DOFj+1,DOFj:DOFj+1)+KGlobal(3:4,3:4);
	KGlobalStructure(DOFi:DOFi+1,DOFj:DOFj+1)=KGlobalStructure(DOFi:DOFi+1,DOFj:DOFj+1)+KGlobal(1:2,3:4);
	KGlobalStructure(DOFj:DOFj+1,DOFi:DOFi+1)=KGlobalStructure(DOFj:DOFj+1,DOFi:DOFi+1)+KGlobal(3:4,1:2);
end


%% Form Rearrangement matrix V
ZeroDisplacements = 0; %Set known (zero) DOF displacements to zero
UnknownDisplacements = 0; %Set unknown DOF displacements to zero
for n=1:(NumDOFs) %Scan all DOFs
	node = fix((n-1)/2)+1; %Node where the DOF belongs to
	DOF_ID = n-2*(node-1); %Assign 1, 2 for x-displacement or y-displacement, respectively
	if BoundaryConditions(node,DOF_ID)==1 %In case of constrained DOF
        ZeroDisplacements = ZeroDisplacements + 1; %Count known DOFs
        V1order(ZeroDisplacements)=n; %order known DOFs assending
	else UnknownDisplacements =UnknownDisplacements + 1; %In case of unconstrained DOF, Count unknown DOFs
        V2order(UnknownDisplacements)=n; %order unknown DOFs assending
	end
end

if UnknownDisplacements==0
    Vorder = V1order;
else
    Vorder = [V2order V1order]; %Put together to form Vorder
end
 
V = zeros(NumDOFs); %Initialize V putting zeros in it
for n=1:NumDOFs; V(n,Vorder(n))=1; end; %Form rearrangement matrix by putting ones into the appropriate positions
    

%% Form rearranged structure's global stiffness matrix (Km)
Km = V*KGlobalStructure*V';
Kff = Km(1:UnknownDisplacements,1:UnknownDisplacements);
Ksf = Km((UnknownDisplacements+1):NumDOFs,1:UnknownDisplacements);


%% Form Nodal loads array LoadVec
LoadVec = reshape(NodalLoads',2*NumNodes,1); %Loads on each Model DOF
LoadVecRearranged = V*LoadVec; %Rearranged load vector


%% Calculate unknown displacements
Pf = LoadVecRearranged(1:UnknownDisplacements);
Df = Kff\Pf; %Solve linear system, Calculate unknown displacements (global system)

DisplVecRearranged = [Df ; zeros(ZeroDisplacements,1)]; %rearranged vector of displacements (global system)
DisplacementsGlobal = V'*DisplVecRearranged; %Vector of displacements (global system)


%% Calculate constraint reactions
ConstraintReactions = Ksf*Df - LoadVecRearranged((UnknownDisplacements+1):NumDOFs); %In case there are forces directly on constrained DOFs


%% Calculate element forces
ElementALLForces = zeros(NumElements,1); %Initialize ElementALLForces putting zeros in it
ElementALLStresses = zeros(NumElements,1); %Initialize ElementALLStresses putting zeros in it
ElementALLStrains = zeros(NumElements,1); %Initialize ElementALLStrains putting zeros in it
for n=1:NumElements %Scan all elements
	PTrussAssignElementProperties  %Get properties for n element
	KLocal = PTrussElementStiffness(E,A,L); %Form local stiffness matrix (KLocal) for n element [2x2]
    Transform = PTrussElementTransformation(x1,y1,x2,y2); %n element transformation matrix [2x4]
	DOFi = 2*Node_i-1 ; %x-Degree OF Freedom for start node (i) of n element
	DOFj = 2*Node_j-1 ; %x-Degree OF Freedom for end node (j) of n element
    ElementnGlobalDisplacements = [DisplacementsGlobal(2*Node_i-1:2*Node_i) ; DisplacementsGlobal(2*Node_j-1:2*Node_j)]; %n element displacements in global system [4x1]
    ElementnLocalDisplacements = Transform*ElementnGlobalDisplacements; %n element displacements in local system [2x1]
    ElementnLocalForces = KLocal*ElementnLocalDisplacements; %n element forces in local system [2x1]
	ElementALLForces(n,:)=ElementnLocalForces(2,1); %Add element contribution to the forces matrix for all elements [NumElements x 1]
    %Note that we only keep the force at end j (end of the element) as its sign shows tension (+) or compression (-) for the whole element
    ElementALLStresses(n,:)=ElementnLocalForces(2,1)/A; %Divide Force by Area to obtain stresses [NumElements x 1]
    ElementALLStrains(n,:)=ElementALLStresses(n,:)/E; %Divide Stress by Modulus of Elasticity to obtain strains [NumElements x 1]
end


%% Do some postprocessing and export analysis results
%Convert displacements to format [NumNodes x 2] where each row is one node
for n=1:NumNodes;
    NodeDisplacementsGlobal(n,1:2)=DisplacementsGlobal(2*n-1:2*n);
end

%Convert constraint reactions to format: [Node DOF Reaction]
nodeIDReaction = fix((V1order' - 1) / 2) + 1; %Node where the DOF belongs to
DOFIDReaction = V1order' - 2 * (nodeIDReaction - 1); %Assign 1, 2 for x-displacement or y-displacement, respectively
NodeDOFReaction = [nodeIDReaction DOFIDReaction ConstraintReactions]; %Put them all together

PTrussDataOutput; %Export analysis results
toc %terminate timer and report elapsed time

% End of file -------------------------------------------------------------